<?php

namespace Drupal\grouper_message_filter\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Grouper Message Filter settings for this site.
 */
class SettingsForm extends ConfigFormBase {
  /**
   * Configuration Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Messanger di Interface.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'grouper_message_filter_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['grouper_message_filter.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['#attached']['library'][] = 'grouper_message_filter/grouper_message_filter';

    $form['#tree'] = TRUE;

    $form['description'] = [
      '#type' => 'item',
      '#title' => $this->t('Currently filtering Log Messages for these message types. For each type, messages can be filtered by severity, level and message text rules.'),
    ];

    // Gather the number of types in the form already.
    $num_types = $form_state->get('num_types');

    // We have to ensure that there is at least one name field.
    if ($num_types === NULL) {

      $this->loadRenderInfo($form_state);
      // Build the number of 'type' fieldsets $form_state['num_types'].
      $num_types = $form_state->get('num_types');

    }

    $rule_index = 0;

    // Build the number of 'type' fieldsets $form_state['num_types'].
    for ($type_index = 1; $type_index <= $num_types; $type_index++) {

      $form['filters'][$type_index] = [
        '#type' => 'fieldset',
        '#title' => 'Filter ' . $type_index,
      ];

      $form['filters'][$type_index]['type'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Message Type'),
        '#default_value' => $this->config('grouper_message_filter.settings')->get('filters')[$type_index]['type'],
      ];

      $num_rules_array = $form_state->get('num_rules_array');
      $num_rules = $num_rules_array[$type_index];

      for ($rule_index = 1; $rule_index <= $num_rules; $rule_index++) {

        $form['filters'][$type_index]['rule'][$rule_index] = [
          '#type' => 'fieldset',
          '#title' => 'Rule ' . $rule_index,
        ];

        // The filter options are offset by 1000 to avoid issues with 0.
        $form['filters'][$type_index]['rule'][$rule_index]['severity'] = [
          '#type' => 'checkboxes',
          '#title' => $this->t('Severity'),
          '#options' => [
            '1000' => $this->t('Emergency'),
            '1001' => $this->t('Alert'),
            '1002' => $this->t('Critical'),
            '1003' => $this->t('Error'),
            '1004' => $this->t('Warning'),
            '1005' => $this->t('Notice'),
            '1006' => $this->t('Info'),
            '1007' => $this->t('Debug'),
          ],
          '#default_value' => $this->config('grouper_message_filter.settings')->get('filters')[$type_index]['rule'][$rule_index]['severity'],
        ];

        $form['filters'][$type_index]['rule'][$rule_index]['message_contains'] = [
          '#type' => 'textfield',
          '#title' => $this->t('And Message Contains'),
          '#default_value' => $this->config('grouper_message_filter.settings')->get('filters')[$type_index]['rule'][$rule_index]['message_contains'],
        ];

      }

      // Adds "Add another rule" button.
      $form['filters'][$type_index]['rule']['add_rule'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add another rule to this type'),
        '#submit' => [[$this, 'addRule']],
        '#name' => "$type_index",
      ];

    }

    // Adds "Add another name" button.
    $form['add_type'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another type'),
      '#submit' => [[$this, 'addType']],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  /*
  public function validateForm(array &$form, FormStateInterface $form_state) {

  if ($form_state->getValue('field') != 'field') {
  $form_state->setErrorByName('field', $this->t('The value is not correct.'));
  }

  parent::validateForm($form, $form_state);
  }
   */

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $values = ($form_state->getValues()['filters'] ?? []);

    if (!empty($values)) {
      // If there are empty types or rules delete them.
      $clean_values = $this->cleanseValues($values);

      $config = $this->config('grouper_message_filter.settings');

      // Save the config data. Pretty easy.
      $config->set('filters', $clean_values)->save();

      $form_state->setRebuild(TRUE);

      /*
      // Redirect to the users page.
      Option $form_state->setRedirect('user.page');
       */

      // Set rebuild.
      $form_state->setRebuild(TRUE);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Adds a new log message type to filter.
   */
  public function addType(array &$form, FormStateInterface $form_state) {

    // Add another type.
    $num_types = $form_state->get('num_types');
    $form_state->set('num_types', $num_types + 1);

    // Add a rule for the new type.
    $num_rules_array = $form_state->get('num_rules_array');
    $num_rules_array[] = 1;
    $form_state->set('num_rules_array', $num_rules_array);

    // Set rebuild.
    $form_state->setRebuild(TRUE);

  }

  /**
   * Adds a new rule or the log message type to filter.
   */
  public function addRule(array &$form, FormStateInterface $form_state) {

    // Gets the name of the Triggering Element which has been set to type id.
    $which_type_id = intval($form_state->GetTriggeringElement()['#name']);

    // Add a rule to the specified type.
    $num_rules_array = $form_state->get('num_rules_array');
    $num_rules_array[$which_type_id]++;
    $form_state->set('num_rules_array', $num_rules_array);

    // Set rebuild.
    $form_state->setRebuild(TRUE);

  }

  /**
   * Gets the number of types and rules in the config to render.
   */
  public function loadRenderInfo(FormStateInterface $form_state) {

    // Get the config data.
    $filters = $this->config('grouper_message_filter.settings')->get('filters');

    // Set the number of Types.
    $form_state->set('num_types', count($filters));

    // Determine the number of Rules foreach Type.
    $index = 0;
    $config_number_of_rules_array = [];
    foreach ($filters as $type) {
      $index++;
      $config_number_of_rules_array[$index] = count($type['rule']) - 1;
    }

    // Set the number of Rules for each type.
    $form_state->set('num_rules_array', $config_number_of_rules_array);

  }

  /**
   * Gets an empty rule array for testing and use.
   */
  public function getEmptyRule() {
    return [
      'severity' => [
        '1000' => 0,
        '1001' => 0,
        '1002' => 0,
        '1003' => 0,
        '1004' => 0,
        '1005' => 0,
        '1006' => 0,
        '1007' => 0,
      ],
      'message_contains' => '',
    ];
  }

  /**
   * Determines if a rule is ineffective.
   *
   * The message_contains has a message fragment in it.
   * But no severity is set so it won't be filtered.
   * Used to notify the user that the rule won't filter anything.
   */
  public function ruleHasContainsButNoSeverity(array $rule) {
    $message_contains = ($rule['message_contains'] ?? '');
    if (!empty($message_contains)) {
      if (isset($rule['severity']) && count($rule['severity'])) {
        foreach ($rule['severity'] as $level) {

          if ($level != 0) {
            return FALSE;
          }

        }
        return TRUE;
      }

    }
    return FALSE;
  }

  /**
   * Cleans the form.
   *
   * If type is empty or if a rule is empty get rid of it.
   * Return a new array without all the empty Types and Rules.
   * If detects a rule whose Message Contains is not empty.
   * But where no severity is set a message to the user is emitted.
   */
  public function cleanseValues(array &$values) {
    $empty_rule = $this->getEmptyRule();
    $clean_settings = [];

    foreach ($values as $index => $type) {
      if (!empty($type['type'])) {
        $clean_rules = [];
        $clean_index = 1;
        // Iterate thru the rules and copy non empty rules.
        foreach ($type['rule'] as $index => $rule) {
          if (is_numeric($index)) {
            // If $rule is NOT empty (is different from $empty_rule).
            if ($rule !== $empty_rule) {
              // The $rule is not empty copy it in.
              $clean_rules[$clean_index++] = $rule;

              // If severity is empty & message_contains is not.
              if ($this->ruleHasContainsButNoSeverity($rule)) {
                $this->messenger->addMessage('A rule has no severity set. And Message Contains is not empty : ' . ($rule['message_contains'] ?? 'something'));
              }
            }
            else {
              // The rule is empty.
              // Only copy it if it's the first ie only item.
              if ($index == 1 && count($type['rule']) == 2) {
                $clean_rules[$clean_index++] = $rule;
              }
            }
          }
          else {
            // The add_rule element.
            $clean_rules['add_rule'] = $rule;
          }

        }

        // Done iterating thru the old $rules and building $clean_rules.
        // Put it all in a new $clean_type and add that to $clean_settings.
        $clean_type = [];
        $clean_type['type'] = $type['type'];
        $clean_type['rule'] = $clean_rules;
        $clean_settings[count($clean_settings) + 1] = $clean_type;

        /*
        \Drupal::messenger()->addMessage(print_r($clean_settings,1));
         */
      }

    }

    return $clean_settings;

  }

}
